#!/bin/bash
if [ "$BASH" != "/bin/bash" ] && [ "$BASH" != "/usr/bin/bash" ]
then
   bash "$0" "$@"
   exit $?
fi

function error(){
    echo -e "\033[1;31m$1\033[0m"
    exit 1
}

CURRENT_DIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
cd "${CURRENT_DIR}" || exit 1

ALL_CPUS=$(cat /sys/devices/system/cpu/online)
MAX_NUM=8
GPU="renderD128"
BINDERNODE=/dev/binder0
HWBINDERNODE=/dev/binder1
VNDBINDERNODE=/dev/binder2
box_data="/var/lib/Kbox"
lxcfs_path="/var/lib/lxcfs"
path_cpu="/sys/devices/system/cpu"
box_name=android_1

function getcpus(){
    local cpus_r
    local start
    local end
    cpus_r=$(echo ${ALL_CPUS}|sed 's/,/ /g')
    for cpu_r in ${cpus_r}
    do
        echo ${cpu_r}|grep "-" > /dev/null
        if [ $? -eq 0 ]; then
            start=$(echo ${cpu_r}|cut -d '-' -f1)
            end=$(echo ${cpu_r}|cut -d '-' -f2)
        else
            start=${cpu_r}
            end=${cpu_r}
        fi
        echo $(seq ${start} 1 ${end})
    done
}

function select_cpus(){
    local s
    local cpus
    local cpu_num
    cpus=($(getcpus))
    cpu_num=${#cpus[*]}
    if [ ${cpu_num} -gt ${MAX_NUM} ]
    then
        s=$((cpu_num - MAX_NUM))
        while (($s < $cpu_num))
        do
            echo "${cpus[s]}"
            (( s++ )) || true
        done
    else
        echo "${cpus[*]}"
    fi
}

function check_modules(){
    local count_time=0
    while true
    do
        if [ -e /dev/binder2 ] && [ -e /dev/ashmem ]; then
            break
        fi
        # 20秒内核模块未注册成功超时退出
        if [ ${count_time} -le 20 ]
        then
            sleep 1
            (( count_time++ )) || true
        else
            error "kernel modules insmod failed"
        fi
    done
}


function import_image(){
    image_id=$(docker images -a -q -f "reference=kbox:10")
    if [ -n "${image_id}" ]; then
        docker rmi ${image_id} || true
    fi
    docker import /var/lib/Kbox/android.tar kbox:10
}

function rm_image(){
    docker rmi kbox:10
}

function is_exist_container(){
    local containerid
    containerid=$(docker ps -a -q -f "name=${box_name}")
    if [ -n "${containerid}" ]; then
        echo "The ${box_name} exists" && return 0
    else
        echo "The ${box_name} does not exist" && return 1
    fi
}

function create_container(){
    local scpus
    local bind_cpu
    local acnt
    local vcpus
    local containerid
    containerid=$(docker ps -a -q -f "name=${box_name}")
    if [ -n "${containerid}" ]; then
        echo "The ${box_name} exists,please rm ${box_name} first." && return 1
    fi
    import_image
    scpus=($(select_cpus))
    bind_cpu=$(echo "${scpus[*]}"|tr ' ' ',')
    acnt=0
    vcpus=""
    for a in ${scpus[*]}
    do
        vcpus=$vcpus" -v ${path_cpu}/cpu${a}:${path_cpu}/cpu${acnt}:rw"
        acnt=$((acnt+1))
    done
    
    [ -d "/var/run/docker/cpus/${box_name}" ] &&  rm -rf "/var/run/docker/cpus/${box_name}"
    mkdir -p /var/run/docker/cpus/"${box_name}"
    docker run -d -it \
        --cap-add=ALL \
        --security-opt="apparmor=unconfined" \
        --security-opt="seccomp=unconfined" \
        --pids-limit=-1 \
        --cpuset-cpus "${bind_cpu}" \
        --hostname "localhost" \
        --name "${box_name}" \
        -e DOCKER_NAME="${box_name}" \
        -e PATH=/system/bin:/system/xbin \
        -e GATEWAY=172.200.0.1 \
        --net PC-Kbox \
        --ip 172.200.0.2 \
        --device=${BINDERNODE}:/dev/binder:rwm \
        --device=${HWBINDERNODE}:/dev/hwbinder:rwm \
        --device=${VNDBINDERNODE}:/dev/vndbinder:rwm \
        --device=/dev/ashmem:/dev/ashmem:rwm \
        --device=/dev/fuse:/dev/fuse:rwm \
        --device=/dev/dri/${GPU}:/dev/dri/renderD128:rwm \
        --device=/dev/uinput:/dev/uinput:rwm \
        --device-cgroup-rule="c 13:* rwm" \
        --volume=${box_data}/cache:/cache:rw \
        --volume=${box_data}/data:/data:rw \
        --volume=/var/run/1/input/event0:/dev/input/event0:rw \
        --volume=/var/run/1/input/event1:/dev/input/event1:rw \
        -v /sys/devices/system/cpu/present:/sys/devices/system/cpu/present \
        -v /sys/devices/system/cpu/possible:/sys/devices/system/cpu/possible \
        -v ${lxcfs_path}/proc/cpuinfo:/proc/cpuinfo:rw \
        -v ${lxcfs_path}/proc/diskstats:/proc/diskstats:rw \
        -v ${lxcfs_path}/proc/meminfo:/proc/meminfo:rw \
        -v ${lxcfs_path}/proc/stat:/proc/stat:rw \
        -v ${lxcfs_path}/proc/swaps:/proc/swaps:rw \
        -v ${lxcfs_path}/proc/uptime:/proc/uptime:rw \
        -v /var/run/docker/cpus/"${box_name}":/sys/devices/system/cpu:rw \
        $vcpus \
        -v $path_cpu/online:$path_cpu/online:rw \
        -v $path_cpu/modalias:$path_cpu/modalias:rw \
        -v $path_cpu/cpufreq:$path_cpu/cpufreq:rw \
        -v $path_cpu/hotplug:$path_cpu/hotplug:rw \
        -v $path_cpu/power:$path_cpu/power:rw \
        -v $path_cpu/uevent:$path_cpu/uevent:rw \
        -v $path_cpu/isolated:$path_cpu/isolated:rw \
        -v $path_cpu/offline:$path_cpu/offline:rw \
        -v $path_cpu/cpuidle:$path_cpu/cpuidle:rw \
        kbox:10  /PC_Kbox_init.sh
}

function stop_container(){
    docker stop ${box_name}
}

function restart_container(){
    docker restart ${box_name}
}

function rm_container(){
    docker rm -f ${box_name}
}

function Status_container(){
    local status
    #running/exited/created 除running状态都返回非0
    status=$(docker inspect -f {{.State.Status}} ${box_name})
    if [ "${status}" = "running" ]
    then
        echo "The ${box_name} is running" && return 0
    else
        echo "The ${box_name} is ${status}" && return 1
    fi
}

function get_ip(){
    local containerip
    containerip=$(docker inspect -f '{{index .NetworkSettings.Networks "PC-Kbox" "IPAddress"}}' ${box_name})
    if [ -n "${containerip}" ]; then
        echo -n "${containerip}" && return 0
    else
        return 1
    fi
}

main(){
    action=$1
    case $action in
        create)
            create_container
        ;;
        stop)
            stop_container
        ;;
        restart)
	    check_modules
            restart_container
        ;;
        rm)
            rm_container
        ;;
        rmi)
            rm_image
        ;;
        status)
            Status_container
        ;;
        is-exist)
            is_exist_container
        ;;
        getip)
            get_ip
        ;;
        *)
            echo "unknown action"
            return 1
        ;;
    esac
}
main "$@"
exit $?
